/* Copyright (c) 1994, 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>
#include <math.h>


#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#include <string.h>

#include "jdmath.h"
#include "_jdmath.h"


int JDM_bilinear_init (JDM_Bilinear_Interp_Type *b,
		       double x, double y,
		       double dx, double dy,
		       unsigned int num_dims,
		       double *f_ll, double *f_lr, double *f_ul, double *f_ur)
{
   unsigned int i;

   if ((b == NULL)
       || (dx == 0.0)
       || (dy == 0.0)
       || (num_dims > JDM_MAX_BILINEAR_DIMS))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }
   
   b->x = x;
   b->y = y;
   b->inv_dx = 1.0 / dx;
   b->inv_dy = 1.0 / dy;
   
   for (i = 0; i < num_dims; i++)
     {
	double lr, ll, ul, ur;
	
	lr = f_lr[i];
	ll = f_ll[i];
	ul = f_ul[i];
	ur = f_ur[i];

	lr -= ll;
	ur -= ll;
	ul -= ll;
   
	b->f0[i] = ll;
	b->f1[i] = lr;
	b->f2[i] = ul;
	b->f3[i] = (ur - lr - ul);
     }
   
   b->num_dims = num_dims;

   return 0;
}


int JDM_bilinear_interp (JDM_Bilinear_Interp_Type *b, double x, double y,
			 double *result)
{
   unsigned int i, num_dims;

   if (b == NULL)
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }
   x = b->inv_dx * (x - b->x);
   y = b->inv_dy * (y - b->y);
   
   num_dims = b->num_dims;

   for (i = 0; i < num_dims; i++)
     result[i] = b->f0[i] + x * (b->f1[i] + y * b->f3[i]) + y * b->f2[i];

   return 0;
}

   
