/* Copyright (c) 1994, 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>
#include <math.h>


#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#include <string.h>

#include "jdmath.h"

/* This function computes a histogram of the data in the @pts@ array.  There
 * are assumed to be @npts@ points in this array.
 */
int JDMhistogram_d (double *pts, unsigned int npts,
		    unsigned int *bin, unsigned int nbin, 
		    double bmin, double bmax)
{
   unsigned int i;
   double dbin;
   
   if ((nbin == 0) 
       || (bmax <= bmin))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }
   
   for (i = 0; i < nbin; i++) bin[i] = 0;
   
   dbin = (bmax - bmin) / nbin;
   
   for (i = 0; i < npts; i++)
     {
	double x;
	unsigned int j;
	
	x = pts[i];
	if ((x < bmin) || (x > bmax)) continue;
	
	j = (unsigned int) ((x - bmin) / dbin);
	
	/* j could be equal to nbin if x == bmax.  Since we are allowing x
	 * to be equal to bmax, do the following: */
	if (j == nbin) j--;
	
	bin[j] += 1;
     }
   
   return 0;
}

	
   
int JDMhistogram_f (float *pts, unsigned int npts,
		    unsigned int *bin, unsigned int nbin, 
		    float bmin, float bmax)
{
   unsigned int i;
   double dbin;
   
   if ((nbin == 0) 
       || (bmax <= bmin))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }

   for (i = 0; i < nbin; i++) bin[i] = 0;
   
   dbin = (bmax - bmin) / nbin;
   
   for (i = 0; i < npts; i++)
     {
	double x;
	unsigned int j;
	
	x = pts[i];
	if ((x < bmin) || (x > bmax)) continue;
	
	j = (unsigned int) ((x - bmin) / dbin);
	
	/* j could be equal to nbin if x == bmax.  Since we are allowing x
	 * to be equal to bmax, do the following: */
	if (j == nbin) j--;
	
	bin[j] += 1;
     }
   
   return 0;
}

unsigned int JDMhist_examine_fdata (float *pts, unsigned int npts, float bin_size,
				    float *xmin, float *xmax)
{
   double x0, x1;
   unsigned int i;

   if ((npts == 0) || (pts == NULL) || (bin_size <= 0.0))
     {
	*xmin = *xmax = 0;
	return 0;
     }
   
   x0 = x1 = pts[0];
   for (i = 1; i < npts; i++)
     {
	double x;
	
	x = pts[i];
	if (x > x1) x1 = x;
	else if (x < x0) x0 = x;
     }
   *xmax = (float) x1;
   *xmin = (float) x0;
   return (x1 - x0) / bin_size;
}

unsigned int JDMhist_examine_ddata (double *pts, unsigned int npts, double bin_size,
				    double *xmin, double *xmax)
{
   double x0, x1;
   unsigned int i;

   if ((npts == 0) || (pts == NULL) || (bin_size <= 0.0))
     {
	*xmin = *xmax = 0;
	return 0;
     }
   
   x0 = x1 = pts[0];
   for (i = 1; i < npts; i++)
     {
	double x;
	
	x = pts[i];
	if (x > x1) x1 = x;
	else if (x < x0) x0 = x;
     }
   *xmax = x1;
   *xmin = x0;
   return (x1 - x0) / bin_size;
}

   
