/* Copyright (c) 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>
#include <string.h>

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#include "jdmath.h"
#include "_jdmath.h"

typedef struct 
{
   unsigned int num_vary;
   unsigned int *parm_index;
   double *tmp_parms;
   double *delta_parms;
   double lambda;
   double last_chisqr;
}
JDMLevMarq_Type;

static void free_lm (JDMLevMarq_Type *lm)
{
   if (lm == NULL) return;
   _JDMfree ((char *) lm->parm_index);
   _JDMfree ((char *) lm->tmp_parms);
   _JDMfree ((char *) lm->delta_parms);
   _JDMfree ((char *) lm);
}


JDMLevMarq_Type *JDMleven_marq_open (unsigned int *vary_list, 
				     unsigned int num_parms,
				     unsigned int num_to_vary)
{
   JDMLevMarq_Type *lm;
   unsigned int i, j;

   if ((num_parms == 0) || (num_to_vary == 0)
       || (num_to_vary > num_parms))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return NULL;
     }

   if (NULL == (lm = (JDMLevMarq_Type *) _JDMmalloc (sizeof (JDMLevMarq_Type),
						     "JDMleven_marq_open")))
     return NULL;
   memset ((char *) lm, 0, sizeof (JDMLevMarq_Type));

   if ((NULL == (lm->parm_index = (unsigned int *) JDMinteger_vector (num_to_vary)))
       || (NULL == (lm->tmp_parms = JDMdouble_vector (num_to_vary)))
       || (NULL == (lm->delta_parms = JDMdouble_vector (num_to_vary))))
     {
	free_lm (lm);
	return NULL;
     }

   lm->num_vary = num_to_vary;

   j = 0;
   for (i = 0; i < num_parms; i++)
     {
	if (vary_list [i] == 0)
	  continue;
	
	lm->parm_index[j] = i;
	j++;
     }

   lambda = 0.001;
   return lm;
}

   
int JDMleven_marq (JDMLevMarq_Type *lm,
		   double *x, double *y, double *dy, unsigned int npts,
		   double *parm_list, double **covar, double **alpha,
