/* Copyright (c) 1994, 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>
#include <math.h>


#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif
#include <string.h>

#include "jdmath.h"

int JDMlog_grid (double *x, unsigned int num, double a, double b)
{
   double ratio, factor;
   unsigned int i;
   
   if ((a <= 0.0) || (a > b) || (num < 2))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }
   
   ratio = b / a;
   factor = 1.0 / (num - 1);
   
   for (i = 0; i < num; i++)
     {
	x[i] = a * pow (ratio, i * factor);
     }
   
   return 0;
}

int JDMlog_grid_f (float *x, unsigned int num, float a, float b)
{
   double ratio, factor;
   unsigned int i;
   
   if ((a <= 0.0) || (a > b) || (num < 2))
     {
	JDMath_Error = JDMATH_INVALID_PARAMETER;
	return -1;
     }
   
   ratio = b / a;
   factor = 1.0 / (num - 1);
   
   for (i = 0; i < num; i++)
     {
	x[i] = (float) (a * pow (ratio, i * factor));
     }
   
   return 0;
}
