/* Copyright (c) 1994, 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"
#include "jdmath.h"

unsigned int JDMcheck_nan (double *x, unsigned int n)
{
#ifdef HAVE_ISNAN
   unsigned int i;
   for (i = 0; i < n; i++)
     {
	if (isnan (x[i]))
	  return i + 1;
     }
#endif
   return 0;
}

unsigned int JDMcheck_inf (double *x, unsigned int n, int *sign)
{
#ifdef HAVE_ISINF
   unsigned int i;
   for (i = 0; i < n; i++)
     {
	if (0 == isinf (x[i]))
	  continue;
	*sign = isinf (x[i]);
	return i + 1;
     }
#endif
   return 0;
}

unsigned int JDMcheck_finite (double *x, unsigned int n)
{
#ifdef HAVE_FINITE
   unsigned int i;
   for (i = 0; i < n; i++)
     {
	if (0 == finite (x[i]))
	  return i + 1;
     }
#else
# if defined(HAVE_ISINF) && defined(HAVE_ISNAN)
   unsigned int i;
   for (i = 0; i < n; i++)
     {
	double xi = x[i];
	if (isinf (xi) || isnan (xi))
	  return i + 1;
     }
# endif   
#endif
   return 0;
}

int JDMisnan (double x)
{
#ifdef HAVE_ISNAN
   return isnan (x);
#else
   return 0;
#endif
}

int JDMisinf (double x)
{
#ifdef HAVE_ISINF
   return isinf (x);
#else
   return 0;
#endif
}

int JDMfinite (double x)
{
#ifdef HAVE_FINITE
   return finite (x);
#else
# if defined(HAVE_ISINF) && defined(HAVE_ISNAN)
   return ((0 == isinf(x)) && (0 == isnan(x)));
# endif
#endif
}
