/* Copyright (c) 1994, 1998 John E. Davis
 * This file is part of the JDMath library.
 *
 * You may distribute this file under the terms the GNU General Public
 * License.  See the file COPYING for more information.
 */
#include "config.h"

#include <stdio.h>


#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#include "jdmath.h"

static void error (char *s)
{
   fprintf (stderr, "\n\n****ERROR: %s\n", s);
   fputs ("\
\n\
An error has been encountered while trying to determine information about\n\
your computer system.  You may have to edit the file jdmath.h to port the\n\
library to your system.\n\n", 
	  stderr);
	  
   exit (-1);
}


int main (int argc, char **argv)
{
   float x;
   unsigned long n;
   unsigned short s;

   (void) argc; (void) argv;
   if (sizeof (int16) != 2) 
     error ("Sizeof int16 != 2");
   if (sizeof (int32) != 4)
     error ("Sizeof int32 != 4");
   if (sizeof (float32) != 4)
     error ("Sizeof float32 != 4");
   if (sizeof (float64) != 8)
     error ("Sizeof float64 != 8");
   
   /* Now determine floating point format and byte swapping */
   
   s = 0x1234;
   if (*(char *) &s == 0x34)
     {
	fprintf (stdout, "#define NEEDS_BYTE_SWAP\n");
     }
   
   /* Now determine the floating point style */
   
   x = 1.2345678;
   
   if (sizeof (long) == 4)
     {
	n = *(unsigned long *) &x;
     }
   else n = (unsigned long) *(unsigned int *) &x;

   if (n == 0x3F9E0651)
     {
	fprintf (stdout, "#define USES_IEEE_FLOAT\n");
     }
   else if (n == 0x0651409E)
     {
	fprintf (stdout, "#define USES_VAX_FLOAT\n");
     }
   else 
     {
        error ("Unknown floating point format.  Not supported.");
     }
   return 0;
}

